#!/usr/bin/Rscript

# GAD PIPELINE #
# run_de_analysis.R
# Description : This script allows user to run limma and edgeR command for DE analysis
# Usage : python Rscript run_de_analysis.R count_matrix design_file output_directory analysis_basename method filter normlization
# Output : tabulated output file with DE results and QC plots
# Requirements : Rscript, limma, EdgeR

# Author : Emilie.Tisserant@u-bourgogne.fr
# Creation Date : 20170331
# last revision date : 20200930
# Known bugs : None


argv <- commandArgs(TRUE)
inputfile <- argv[1]
designfile <- argv[2]
basename <- argv[3]
outputdir <- argv[4]
method <- argv[5]
filter <- argv[6]
norm <- argv[7]


# method=<limma|edgerqlf|edgerlrt>
# filter= min nb samples with cpm > 1
# norm=<TMM|RLE|upperquartile>

#R_LIBS="/work/gad/shared/bin/R_libs"
#export R_LIBS
#/soft/R/3.2.3/intel/13.1.3/bin/Rscript run_de_analysis.R all.count all.design all /work/gad/em0342ti/RNASeq_pipeline/dataset1/limma/ limma 1 TMM
#/soft/R/3.2.3/intel/13.1.3/bin/Rscript run_de_analysis.R all.count all.design all /work/gad/em0342ti/RNASeq_pipeline/dataset1/edgerqlf/ edgerqlf 1 TMM
#/soft/R/3.2.3/intel/13.1.3/bin/Rscript run_de_analysis.R all.count all.design all /work/gad/em0342ti/RNASeq_pipeline/dataset1/edgerlrt/ edgerlrt 1 TMM

# TODO: LOGGING: sink(file=file.path(outputdir, "log.txt"))
# TODO: automatically assign sample name with design file
# TODO: Genes annotation
# TODO: Data transformation (cpm, lcpm, rpkm ; calcNormFactors()?)
# TODO: Alt design: design <- model.matrix(~0+group,data=data$samples)
# TODO: Alt design: cont.matrix <- makeContrasts(CIvsC=CI-C, levels=design)
# TODO: Alt design: fit<- contrasts.fit(fit, cont.matrix)
# TODO: Blocking design
# TODO: Filter cutoff: option
# TODO: Add color and other dim for MDSplot
# TODO: Add Hierarchical clustering
# TODO: EdgeR Generalized Linear Model (GLM) likelihood ratio test: fit <- glmFit(data,design)
# TODO: EdgeR Generalized Linear Model (GLM) likelihood ratio test: de <- glmLRT(fit)
# TODO: Treat method: calculate p-values with a minimum log-FC requirement) : tfit <- treat(vfit, lfc=1)
# TODO: Treat method limma:  tfit <- treat(vfit, lfc=1)
# TODO: Treat method edgeR:  de <- glmTreat(fit, coef=2, lfc=1)
# TODO: Heatmap
# TODO: Gene set testing


# Logging
sink(file=paste(outputdir, basename, ".log.txt", sep = ""))

# Load package
library(edgeR)
library(methods)

# Read data
rawdata <- read.delim(inputfile,row.names="Genes")
rawdesign <- read.delim(designfile,header=FALSE)
group <- factor(rawdesign$V2)

# Create DGEList
data <- DGEList(counts=rawdata,group=group)

# Build design
design <- model.matrix(~group,data=data$samples)
rownames(design) <- colnames(rawdata)
colnames(design) <- levels(data$samples$group)
rownames(design) <- colnames(rawdata)

# LOG: design and raw data summary
print("Design:")
design
print("Raw data summary:")
data$samples
dim(data)

# PLOT: density for raw data
pdf(paste(outputdir, basename, ".density_raw.pdf", sep = ""),width = 5,height = 5)
plotDensities(cpm(data, log=TRUE), main="Raw data")
dev.off()

# PLOT: boxplot for raw data
pdf(paste(outputdir, basename, ".boxplot_raw.pdf", sep = ""),width = 5,height = 5)
boxplot(cpm(data, log=TRUE), main="Raw data", ylab="Log-cpm",las=2)
dev.off()

# Filtering data
keep <- rowSums(cpm(data)>1) >= filter
data <- data[keep, , keep.lib.sizes=FALSE]

# Recompute the library sizes after filtering
data$samples$lib.size <- colSums(data$counts)

# Calculate the normalization factors for composition bias (TMM)
data <- calcNormFactors(data,method=norm)

# LOG: normalized data summary
print("Normalized data summary:")
data$samples
dim(data)

# PLOT: density for normalized data
pdf(paste(outputdir, basename, ".density_normalized.pdf", sep = ""),width = 10,height = 10)
plotDensities(cpm(data, log=TRUE), main="Normalized data" , legend = "topright")
dev.off()

# PLOT: boxplot for normalized data
pdf(paste(outputdir, basename, ".boxplot_normalized.pdf", sep = ""),width = 10,height = 10)
boxplot(cpm(data, log=TRUE), main="Normalized data", ylab="Log-cpm",las=2 , legend = "topright")
dev.off()

# PLOT: Multi-Dimensional Scaling (MDS) plot
pdf(paste(outputdir, basename, ".plotMDS.pdf", sep = ""),width = 10,height = 10)
plotMDS(cpm(data, log=TRUE), main="MDS Plot", cex=0.5, legend = "topright")
dev.off()

# limma method
if(method == "limma"){
    # Removing heteroscedascity from data (voom)
    v <- voom(data, design, plot=FALSE, save.plot=TRUE)

    # PLOT: Voom plot
    pdf(paste(outputdir, basename, ".voom.pdf", sep = ""),width = 5,height = 5)
    plot(v$voom.xy,xlab="log2(count size+0.5)",ylab="Sqrt(standard deviation)",pch=16,cex=0.25)
    title("voom: Mean-variance trend")
    lines(v$voom.line,col="red")
    dev.off()

    # Fit linear models
    fit <- lmFit(v, design)

    # Test for differential expression
    de <- eBayes(fit)

    # LOG: DE analysis summary
    print("DE analysis summary:")
    print(summary(decideTests(de[,ncol(design)],adjust.method="none")))
    print("DE analysis summary with fdr:")
    print(summary(decideTests(de[,ncol(design)])))

    # PLOT: Sigma vs Average log expression (SA) plot for linear model
    pdf(paste(outputdir, basename, ".plotSA.pdf", sep = ""),width = 10,height = 10)
    plotSA(de, main="Final model: Mean-variance trend", legend = "topright")
    dev.off()

    # PLOT: Mean-difference plot (MD) plot (MA plot)
    pdf(paste(outputdir, basename, ".plotMA.pdf", sep = ""),width = 10,height = 10)
    plotMD(de,coef=ncol(design),status=decideTests(de[,ncol(design)]),hl.cex=0.4, legend = "topright")
    dev.off()

    # Output results
    de.results <- topTable(de, coef=ncol(design), n="Inf",sort.by="P")
    ro.count <- match(rownames(de.results),rownames(data$counts))
    cpm <- cpm(data)
    ro.cpm <- match(rownames(de.results),rownames(cpm))
    comb.results <- cbind(de.results , "UpDown" = decideTests(de[,ncol(design)])[ro.count] ,data$counts[ro.count, ], cpm[ro.cpm, ])
    write.table(comb.results, file=paste(outputdir, basename, ".DE.tsv", sep = ""), sep = "\t" , row.names = TRUE)
}

# edgeR method
if(method == "edgerqlf" | method == "edgerlrt"){
    # Estimate dispersions
    data <- estimateDisp(data,design)

    # PLOT: Biological coefficient of variation (BCV) plot
    pdf(paste(outputdir, basename, ".plotBCV.pdf", sep = ""),width = 10,height = 10)
    plotBCV(data, main="BCV Plot", legend = "topright")
    dev.off()

    # Testing for differential expression
    # Quasi-Likelihood (QL) F-test
    if(method == "edgerqlf"){
        fit <- glmQLFit(data,design)
        de <- glmQLFTest(fit)
    }
    # Generalized Linear Model (GLM) likelihood ratio test:
    if(method == "edgerlrt"){
        fit <- glmFit(data,design)
        de <- glmLRT(fit)
    }

    # LOG: DE analysis summary
    print("DE analysis summary:")
    print(summary(decideTestsDGE(de,adjust.method="none")))
    print("DE analysis summary with fdr:")
    print(summary(decideTestsDGE(de)))

    # PLOT: MA plot
    pdf(paste(outputdir, basename, ".plotMA.pdf", sep = ""),width = 10,height = 10)
    detags <- rownames(data)[as.logical(decideTestsDGE(de))]
    plotSmear(de, de.tags=detags, main="MA Plot", legend = "topright")
    abline(h=c(-1, 1), col="blue")
    dev.off()

    # Output Results
    de.results <- topTags(de,n=nrow(de$table),sort.by="P")$table
    ro.count <- match(rownames(de.results),rownames(data$counts))
    cpm <- cpm(data)
    ro.cpm <- match(rownames(de.results),rownames(cpm))
    comb.results <- cbind(de.results,"UpDown" = decideTestsDGE(de)[ro.count] ,data$counts[ro.count, ], cpm[ro.cpm, ])
    write.table(comb.results, file=paste(outputdir, basename, ".DE.tsv", sep = ""), sep = "\t" , row.names = FALSE , quote = TRUE, col.names = TRUE )
}
