#!/bin/ksh

### GAD PIPELINE ###
## add_readgroups.sh
## Description : This script allows user to launch Picard AddOrReplaceReadGroups command
## Usage : qsub -pe smp 1 -v INPUTFILE=</path/to/the/input/file>,OUTPUTFILE=</path/to/the/output/file>,TECHNOLOGY=<wes|miseq>,[BATCHNAME=<batch name],[LOGFILE=/path/to/the/log/file],[CONFIGFILE=/path/to/the/config/file] add_readgroups.sh
## Output : BAM output file
## Requirements : jdk 1.8.0, Picard 2.4.1

## Author : yannis.duffourd@u-bourgogne.fr
## Creation Date : 20170327
## last revision date : 20180802
## Known bugs : None

#$ -q batch
#$ -V
#$ -l mem_total=8g

sample=`basename ${INPUTFILE%%.*}`
proc=$(dirname ${INPUTFILE})"/proc"

# Log file path option
if [ -z ${LOGFILE} ]
then
    LOGFILE=add_readgroups.$(date +"%F_%H-%M-%S").log
fi

# Config file path option
if [ -z ${CONFIGFILE} ]
then
    CONFIGFILE=analysis_config.tsv
fi

JAVA=$(grep "javacmd" $CONFIGFILE | awk '{print $2" "$3}')
PICARD=$(grep "picardbase" $CONFIGFILE | awk '{print $2}')
TEMPORARY_DIR=$(grep "temporary_dir" $CONFIGFILE | awk '{print $2}')
PATHSCRIPT=$(grep "pipelinebase" $CONFIGFILE | awk '{print $2}')
PYTHONBIN=$(grep "pythonbin" $CONFIGFILE | awk '{print $2}')


# Logging
exec 1>> $LOGFILE 2>&1
echo "$(date +"%F_%H-%M-%S"): START"

# Check if config file exist
if [ ! -f $CONFIGFILE ]
then
    echo "Config file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch $proc/add_readgroups.failed
    exit 1
fi

# Check if input file exist
if [ ! -f $INPUTFILE ]
then
    echo "Input file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch $proc/add_readgroups.failed
    exit 1
fi

# Check if output file is specified
if [ -z $OUTPUTFILE ]
then
    echo "Output file is not specified"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch $proc/add_readgroups.failed
    exit 1
fi

# Check if technology is specified
if [ -z $TECHNOLOGY ]
then
    echo "Technology is not specified"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch $proc/add_readgroups.failed
    exit 1
fi

# Check if batch name is specified
if [ $TECHNOLOGY == "miseq" ] && [ -z $BATCHNAME ]
then
    echo "Batch name is not specified"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch $proc/add_readgroups.failed
    exit 1
fi

# is now considered as running 


# Launch compile_readgroup.py to compile read group header and check compile_readgroup.py exit code
if [ $TECHNOLOGY == "wes" ]
then
    echo "collect_readgroup.py command: python $PATHSCRIPT/common/fastq/collect_readgroup.py -s $sample -t $TECHNOLOGY"
    readgroup=$($PYTHONBIN $PATHSCRIPT/common/fastq/collect_readgroup.py -s $sample -t $TECHNOLOGY)
    readgroup_exitcode=$?
elif [ $TECHNOLOGY == "miseq" ]
then
    echo "collect_readgroup.py command: python $PATHSCRIPT/common/fastq/collect_readgroup.py -s $sample -t $TECHNOLOGY -b $BATCHNAME"
    readgroup=$($PYTHONBIN $PATHSCRIPT/common/fastq/collect_readgroup.py -s $sample -t $TECHNOLOGY -b $BATCHNAME)
    readgroup_exitcode=$?
fi


ID=$(echo $readgroup | cut -d ":" -f 2 | sed 's/\t.*//g' )
SM=$(echo $readgroup | cut -d ":" -f 3 | sed 's/\t.*//g' )
DS=$(echo $readgroup | cut -d ":" -f 4 | sed 's/\t.*//g' )
PL=$(echo $readgroup | cut -d ":" -f 5 | sed 's/\t.*//g' )
PM=$(echo $readgroup | cut -d ":" -f 6 | sed 's/\t.*//g' )
CN=$(echo $readgroup | cut -d ":" -f 7 | sed 's/\t.*//g' )
LB=$(echo $readgroup | cut -d ":" -f 8)

echo "@RG : ID : $ID ; SM : $SM ; DS : $DS ; PL : $PL ; PM : $PM ; CN : $CN ; LB : $LB" 


# Launch Picard AddOrReplaceReadGroups command and check exit code
echo "Picard command: $JAVA -Djava.io.tmpdir=$TEMPORARY_DIR -Xmx8g -jar $PICARD/picard.jar AddOrReplaceReadGroups \
    INPUT=$INPUTFILE \
    OUTPUT=$OUTPUTFILE \
    RGID=\"$ID\" \
    RGLB=\"$LB\" \
    RGSM=\"$SM\" \
    RGPL=\"$PL\" \
    RGDS=\"$DS\" \
    RGCN=\"$CN\" \
    RGPM=\"$PM\"  \
    RGPU="GAD" \
    CREATE_INDEX=true \
    TMP_DIR=$TEMPORARY_DIR"

$JAVA -Djava.io.tmpdir=$TEMPORARY_DIR -Xmx8g -jar $PICARD/picard.jar AddOrReplaceReadGroups \
    INPUT=$INPUTFILE \
    OUTPUT=$OUTPUTFILE \
    RGID="$ID" \
    RGLB="$LB" \
    RGSM="$SM" \
    RGPL="$PL" \
    RGDS="$DS" \
    RGCN="$CN" \
    RGPM="$PM"  \
    RGPU="GAD" \
    CREATE_INDEX=true \
    TMP_DIR=$TEMPORARY_DIR

addreadgroups_exitcode=$?
echo "addreadgroups exit code : $addreadgroups_exitcode"
if [ $addreadgroups_exitcode != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch add_readgroups.failed
    exit 1
fi

echo "$(date +"%F_%H-%M-%S"): END"


























